/*
 * Copyright (C) 2000-2005 LAAS/CNRS
 *
 * Anthony Mallet - Wed Jun  7 2000
 *
 *
 *
 */

#ifndef _POMSTRUCT_H
#define _POMSTRUCT_H

typedef enum POM_FRAME {
  POM_FRAME_SENSOR,
  POM_FRAME_ROBOT,   /* also known as "main frame" */
  POM_FRAME_BASE,    /* also known as "odometry frame" */
  POM_FRAME_ORIGIN    /* global reference frame */
} POM_FRAME;

typedef struct POM_UNC {
  double utheta,uphi,upsi,ux,uy,uz;
} POM_UNC;

typedef enum POM_ACTION {
   POM_START, POM_STOP, POM_QUERY_MODE
} POM_ACTION;

#define POM_MAX_STRLEN		512
typedef struct POM_STRING {
   char name[POM_MAX_STRLEN];
} POM_STRING;
/*
#define POMDEF_MODEL_FILE	"/usr/local/robots/b21r/modules/pom/models/b21r.geom"
#define POMDEF_LOG_FILE		"bley:/local/images/pom.log.gz"
#define POMDEF_DISPLAYHOST_NAME	"bley"
*/

typedef struct POM_STRING_2 {
   char name1[POM_MAX_STRLEN];
   char name2[POM_MAX_STRLEN];
} POM_STRING_2;

#define POM_DEFAULT_FUSION_METHOD POM_MAX_CONF

typedef enum POM_FUSION_FLAG {
  POM_READ_ONLY, POM_FUSE
} POM_FUSION_FLAG;

/* Motion estimator parameters --------------------------------------- */

typedef struct POM_ME_CREATE_DATA {
  char name[POM_MAX_STRLEN];
  POM_FUSION_FLAG fuseFlag;
  int unused;

#define POMDEF_ME_NAME		""
#define POMDEF_ME_FUSE		POM_FUSE

} POM_ME_CREATE_DATA;

/* Sensor estimator parameters --------------------------------------- */

typedef struct POM_SE_CREATE_DATA {
  char nameposter[POM_MAX_STRLEN];
  char namese[POM_MAX_STRLEN];
  char namefrom[POM_MAX_STRLEN];
  char nameto[POM_MAX_STRLEN];

#define POMDEF_SE_POSTER	""
#define POMDEF_SE_NAME		""
#define POMDEF_SE_FROM		""
#define POMDEF_SE_TO		""

} POM_SE_CREATE_DATA;

/* Reference position ------------------------------------------------ */

typedef struct POM_SET_POS {
   char frame[POM_MAX_STRLEN];	/* internal frame for position */
   double yaw;					/* position (euler) */
   double pitch;
   double roll;
   double x;
   double y;
   double z;

#define POMDEF_POS_FRAME	"Robot"
#define POMDEF_POS_YAW		0.0
#define POMDEF_POS_PITCH	0.0
#define POMDEF_POS_ROLL		0.0
#define POMDEF_POS_X		0.0
#define POMDEF_POS_Y		0.0
#define POMDEF_POS_Z		0.0

} POM_SET_POS;

/* --- Fusion Methods ------------------------------------------------ */

/* Class of fusion methods */
typedef enum POM_FUSION_METHOD {
  POM_FUSION_KALMAN, POM_FUSION_MAX_CONF, POM_FUSION_BAYES
} POM_FUSION_METHOD;

/* --- Euler Pos ------------------------------------------------------ */

typedef struct POM_EULER {
  double yaw, pitch, roll;
  double x, y, z;
} POM_EULER;

typedef struct POM_EULER_VARIANCES {
  double voo;                           /*  0 */
  double vop, vpp;                      /*  1,  2 */
  double vor, vpr, vrr;                 /*  3,  4,  5 */
  double vox, vpx, vrx, vxx;            /*  6,  7,  8,  9 */
  double voy, vpy, vry, vxy, vyy;       /* 10, 11, 12, 13, 14 */
  double voz, vpz, vrz, vxz, vyz, vzz;  /* 15, 16, 17, 18, 19, 20 */
} POM_EULER_VARIANCES;

typedef struct POM_EULER_V {
  POM_EULER euler;
  POM_EULER_VARIANCES var;
} POM_EULER_V;


/* --- Sensor Pos ------------------------------------------------ */

typedef struct POM_SENSOR_POS {
  int	date;		/* time of acquisition */
  int	pad;

  POM_EULER_V sensorToMain;	/* transformation from the sensor's frame to the
			 * main robot's frame */

  POM_EULER_V mainToBase;	/* transformation  from the robot's main frame to
			 * the current local frame */
  POM_EULER_V mainToOrigin;	/* transformation  from the robot's main frame to
				 * the global absolute frame */

  POM_EULER_V VLocal;  /* Local velocities */

} POM_SENSOR_POS;


/** Poster exported by sensor estimators */
typedef struct POM_SE_POSTER {
  char seName[POM_MAX_STRLEN];
  POM_EULER_V seConfig;
} POM_SE_POSTER;


/* --- ME Pos ------------------------------------------------ */

typedef enum POM_ME_CLASS {
   POM_ME_ABSOLUTE, POM_ME_DELTA
} POM_ME_CLASS;

/* Structure that must be produced by a motion estimator */
typedef struct POM_ME_POS {
  POM_ME_CLASS kind;	/* kind of position estimation method */
  int unused;
  double confidence;	/* must be in [0-1] */
  int date1;
  int date2; /* for delta MEs  / xxx to disapear */
  POM_EULER_V main; /* mainToOrigin for absolute MEs, current to previous for delta Mes */
  POM_EULER_V VLocal;
} POM_ME_POS;


/* Similar to previous one but without the uncertainties */
typedef struct POM_EULER_POS {
  int date;
  int unused;
  double confidence;	/* must be in [0-1] */
  POM_EULER main;       /* mainToOrigin */
  POM_EULER VLocal;
} POM_EULER_POS;


/* --- Display params ------------------------------------------------ */

/* typedef struct PomDisplayParams { */
/*    char server[POM_MAX_STRLEN]; */

/* #define POMDEF_SERVER	"farabi" */

/* } PomDisplayParams; */


typedef struct POM_POS {
  int date;
  int pomTickDate; /* for now date = pomTickDate */
  POM_EULER_V mainToOrigin;
  POM_EULER_V mainToBase;
  POM_EULER_V VLocal;
} POM_POS;


typedef struct POM_OBJECT_POS {
  POM_FRAME frame;
  int confidence;      /* 0: not to be used  1: best */
  POM_EULER_V pos;
  POM_EULER_V speed;
  POM_SENSOR_POS sensorPos; /* for now sensor on platine */
} POM_OBJECT_POS;


#endif /* _POMSTRUCT_H */
